@@ -39,7 +39,7 @@
  Michael Emmel
 */
 
-#include <config.h>
+#include "config.h"
 #include <string.h> 
 #include <stdlib.h> 
 #include "gdk.h"
