@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 
 #include "gdkdisplay.h"
 #include "gdkkeys.h"
@@ -33,40 +33,13 @@
 enum {
   DIRECTION_CHANGED,
   KEYS_CHANGED,
+  STATE_CHANGED,
   LAST_SIGNAL
 };
 
-static void gdk_keymap_class_init (GdkKeymapClass *klass);
-
 static guint signals[LAST_SIGNAL] = { 0 };
 
-GType
-gdk_keymap_get_type (void)
-{
-  static GType object_type = 0;
-
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-      {
-        sizeof (GdkKeymapClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) gdk_keymap_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (GdkKeymap),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) NULL,
-      };
-      
-      object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "GdkKeymap",
-                                            &object_info, 0);
-    }
-  
-  return object_type;
-}
+G_DEFINE_TYPE (GdkKeymap, gdk_keymap, G_TYPE_OBJECT)
 
 static void
 gdk_keymap_class_init (GdkKeymapClass *klass)
@@ -77,13 +50,13 @@ gdk_keymap_class_init (GdkKeymapClass *klass)
    * GdkKeymap::direction-changed:
    * @keymap: the object on which the signal is emitted
    * 
-   * The ::direction_changed signal gets emitted when the direction of
+   * The ::direction-changed signal gets emitted when the direction of
    * the keymap changes. 
    *
    * Since: 2.0
    */
   signals[DIRECTION_CHANGED] =
-    g_signal_new ("direction_changed",
+    g_signal_new ("direction-changed",
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GdkKeymapClass, direction_changed),
@@ -95,13 +68,13 @@ gdk_keymap_class_init (GdkKeymapClass *klass)
    * GdkKeymap::keys-changed:
    * @keymap: the object on which the signal is emitted
    *
-   * The ::keys_changed signal is emitted when the mapping represented by
+   * The ::keys-changed signal is emitted when the mapping represented by
    * @keymap changes.
    *
    * Since: 2.2
    */
   signals[KEYS_CHANGED] =
-    g_signal_new ("keys_changed",
+    g_signal_new ("keys-changed",
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GdkKeymapClass, keys_changed),
@@ -109,6 +82,31 @@ gdk_keymap_class_init (GdkKeymapClass *klass)
 		  g_cclosure_marshal_VOID__VOID,
 		  G_TYPE_NONE,
 		  0);
+
+  /**
+   * GdkKeymap::state-changed:
+   * @keymap: the object on which the signal is emitted
+   *
+   * The ::state-changed signal is emitted when the state of the
+   * keyboard changes, e.g when Caps Lock is turned on or off.
+   * See gdk_keymap_get_caps_lock_state().
+   *
+   * Since: 2.16
+   */
+  signals[STATE_CHANGED] =
+    g_signal_new ("state_changed",
+                  G_OBJECT_CLASS_TYPE (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GdkKeymapClass, state_changed),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 
+                  0);
+}
+
+static void
+gdk_keymap_init (GdkKeymap *keymap)
+{
 }
 
 /* Other key-handling stuff
