@@ -24,9 +24,13 @@
  * Modified by the GTK+ Team and others 1997-2001.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_ACCEL_LABEL_H__
 #define __GTK_ACCEL_LABEL_H__
 
@@ -34,10 +38,7 @@
 #include <gtk/gtklabel.h>
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GTK_TYPE_ACCEL_LABEL		(gtk_accel_label_get_type ())
 #define GTK_ACCEL_LABEL(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_ACCEL_LABEL, GtkAccelLabel))
@@ -54,13 +55,13 @@ struct _GtkAccelLabel
 {
   GtkLabel label;
 
-  guint	         queue_id;
-  guint	         accel_padding;
-  GtkWidget     *accel_widget;
-  GClosure	*accel_closure;
-  GtkAccelGroup *accel_group;
-  gchar         *accel_string;
-  guint16        accel_string_width;
+  guint          GSEAL (gtk_reserved);
+  guint          GSEAL (accel_padding);      /* should be style property? */
+  GtkWidget     *GSEAL (accel_widget);       /* done*/
+  GClosure      *GSEAL (accel_closure);      /* has set function */
+  GtkAccelGroup *GSEAL (accel_group);        /* set by set_accel_closure() */
+  gchar         *GSEAL (accel_string);       /* has set function */
+  guint16        GSEAL (accel_string_width); /* seems to be private */
 };
 
 struct _GtkAccelLabelClass
@@ -97,9 +98,11 @@ void	   gtk_accel_label_set_accel_closure (GtkAccelLabel *accel_label,
 					      GClosure	    *accel_closure);
 gboolean   gtk_accel_label_refetch           (GtkAccelLabel *accel_label);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+/* private */
+gchar *    _gtk_accel_label_class_get_accelerator_label (GtkAccelLabelClass *klass,
+							 guint               accelerator_key,
+							 GdkModifierType     accelerator_mods);
 
+G_END_DECLS
 
 #endif /* __GTK_ACCEL_LABEL_H__ */
